const jq = require('node-jq');
const { env, drgWithAttach } = require('./util');

// BTR Data Access calls BTR Services in DRG and returns specified results

async function HasTreatmentWill(attach, PatientCPR) {
    const body = {
        id: 'btr.HasTreatmentWill',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let hasTreatmentWillResponse = await drgWithAttach(attach, body);

    // Returning the boolean telling if there is a treatment will or not
    return hasTreatmentWillResponse.body.response.main.Envelope.Body
        .HasTreatmentWillResponse.willExists._text;
}

async function GetTreatmentWill(attach, PatientCPR) {
    const body = {
        id: 'btr.GetTreatmentWill',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let treatmentWillResponse = await drgWithAttach(attach, body);

    // Returning treatment will component
    return treatmentWillResponse.body.response.main.Envelope.Body
        .GetTreatmentWillResponse.ClinicalDocument.component.structuredBody
        .component.section.entry.observation.value;
}

async function DeleteTreatmentWill(attach, PatientCPR) {
    const body = {
        id: 'btr.DeleteTreatmentWill',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    await drgWithAttach(attach, body);
}

async function CreateTreatmentWill(attach, PatientCPR) {
    const body = {
        id: 'btr.CreateTreatmentWill',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
            noLifeProlongingIfDying: 'false',
            noLifeProlongingIfSeverelyDegraded: 'false',
            noLifeProlongingIfSeverePain: 'true',
            noForcedTreatmentIfIncapable: 'true',
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    await drgWithAttach(attach, body);
}

async function CreateTreatmentWillAdmin(attach, PatientCPR) {
    const body = {
        id: 'btrAdmin.CreateTreatmentWill',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
            noLifeProlongingIfDying: 'true',
            noLifeProlongingIfSeverelyDegraded: 'true',
            noLifeProlongingIfSeverePain: 'true',
            noForcedTreatmentIfIncapable: 'true',
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function DeleteTreatmentWillAdmin(attach, PatientCPR) {
    const body = {
        id: 'btrAdmin.DeleteTreatmentWill',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function UpdateTreatmentWillAdmin(attach, PatientCPR) {
    const body = {
        id: 'btrAdmin.UpdateTreatmentWill',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
            noLifeProlongingIfDying: 'false',
            noLifeProlongingIfSeverelyDegraded: 'true',
            noLifeProlongingIfSeverePain: 'false',
            noForcedTreatmentIfIncapable: 'false',
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function GetTreatmentWillAdmin(attach, PatientCPR) {
    const body = {
        id: 'btrAdmin.GetTreatmentWill',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,

        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

module.exports = {
    HasTreatmentWill,
    GetTreatmentWill,
    DeleteTreatmentWill,
    CreateTreatmentWill,
    CreateTreatmentWillAdmin,
    DeleteTreatmentWillAdmin,
    UpdateTreatmentWillAdmin,
    GetTreatmentWillAdmin,
};
