const {
    default: TestCaseHookDefinition,
} = require('@cucumber/cucumber/lib/models/test_case_hook_definition');
const jq = require('node-jq');
const { env, drgWithAttach } = require('./util');

// DDS Data Access calls DDS Services in DRG and returns specified results

// To get PDC entries to services are called
async function getPDCEntries(attach, PatientCPR) {
    // First calling DDS Registry IDWS
    const body = {
        id: 'ddsregistry.idws',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    // Saving response
    let DDSRegistry = await drgWithAttach(attach, body);

    // Finding entries/documents in response
    let documentlist =
        DDSRegistry.body.response.main.Envelope.Body.AdhocQueryResponse
            .RegistryObjectList.ExtrinsicObject;

    // Finding PDC entry/document in document list
    const res = await jq.run(
        `.[]|select(.Classification[]|select(.__nodeRepresentation== "PDC"))`,
        documentlist,
        { input: 'json', output: 'json' }
    );

    // Saving home community, document unique and repository id from document
    let homeCommunityId = res.__home;
    let documentUniqueId = res.__id;
    let repositoryUniqueId = res.__id.substring(
        0,
        documentUniqueId.indexOf('^')
    );

    // Body to calling DGWS
    const bodyDdsRepository = {
        id: 'ddsrepository.dgws',
        role: 'test.person',
        endpoint: env,
        fields: {
            CitizenCivilRegistrationNumber: PatientCPR,
            homeCommunityId: homeCommunityId,
            repositoryUniqueId: repositoryUniqueId,
            documentUniqueId: documentUniqueId,
        },
    };

    let DDSRepository = await drgWithAttach(attach, bodyDdsRepository);

    let entry_ddsRepositry =
        DDSRepository.body.response.attachments[0].ClinicalDocument.component
            .structuredBody.component.section.entry;

    return entry_ddsRepositry;
}

async function getDDSRegistryWithMoces(attach, Role, PatientCPR) {
    const body = {
        id: 'ddsregistry.dgws',
        role: Role,
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            CitizenCivilRegistrationNumber: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    return await drgWithAttach(attach, body);
}

async function getDDSRegistryWithPoces(attach, PatientCPR) {
    const body = {
        id: 'ddsregistry.idws',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    return await drgWithAttach(attach, body);
}

module.exports = {
    getPDCEntries,
    getDDSRegistryWithMoces,
    getDDSRegistryWithPoces,
};
