const { env, drgWithAttach } = require('./util');

async function CreateOrganDonorRegistration(attach, PatientCPR) {
    const body = {
        id: 'odr.CreateOrganDonorRegistration',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
            permissionType: 'LIMITED_WITH_RESEARCH',
            permissionForHeart: 'true',
            permissionForLungs: 'true',
            permissionForLiver: 'true',
            permissionForPancreas: 'true',
            permissionForKidneys: 'true',
            permissionForCornea: 'true',
            permissionForSmallIntestine: 'true',
            permissionForSkin: 'true',
            requiresRelativeAcceptance: 'true',
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    await drgWithAttach(attach, body);
}

async function GetOrganDonorRegistration(attach, PatientCPR) {
    const body = {
        id: 'odr.GetOrganDonorRegistration',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let organDonorRegistrationResponse = await drgWithAttach(attach, body);

    return organDonorRegistrationResponse.body.response.main.Envelope.Body
        .GetOrganDonorRegistrationResponse.ClinicalDocument.component
        .structuredBody.component.section.entry.observation.value;
}

async function HasOrganDonorRegistration(attach, PatientCPR) {
    const body = {
        id: 'odr.HasOrganDonorRegistration',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let hasOrganDonorRegistrationResponse = await drgWithAttach(attach, body);

    return hasOrganDonorRegistrationResponse.body.response.main.Envelope.Body
        .HasOrganDonorRegistrationResponse.registrationExists._text;
}

async function DeleteOrganDonorRegistration(attach, PatientCPR) {
    const body = {
        id: 'odr.DeleteOrganDonorRegistration',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    await drgWithAttach(attach, body);
}

async function GetOrganDonorRegistrationAdmin(attach, PatientCPR) {
    const body = {
        id: 'odrAdmin.GetOrganDonorRegistration',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    let GetorganDonorRegistrationResponse = await drgWithAttach(attach, body);

    return GetorganDonorRegistrationResponse.body.response.main.Envelope.Body
        .GetOrganDonorRegistrationResponse.ClinicalDocument.component
        .structuredBody.component.section.entry.observation.value;
}

async function CreateOrganDonorRegistrationAdmin(attach, PatientCPR) {
    const body = {
        id: 'odrAdmin.CreateOrganDonorRegistration',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
            permissionType: 'LIMITED_WITH_RESEARCH',
            permissionForHeart: 'true',
            permissionForLungs: 'true',
            permissionForLiver: 'true',
            permissionForPancreas: 'true',
            permissionForKidneys: 'true',
            permissionForCornea: 'true',
            permissionForSmallIntestine: 'true',
            permissionForSkin: 'true',
            requiresRelativeAcceptance: 'true',
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function UpdateOrganDonorRegistrationAdmin(attach, PatientCPR) {
    const body = {
        id: 'odrAdmin.UpdateOrganDonorRegistration',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
            permissionType: 'LIMITED',
            permissionForHeart: 'false',
            permissionForLungs: 'false',
            permissionForLiver: 'true',
            permissionForPancreas: 'false',
            permissionForKidneys: 'true',
            permissionForCornea: 'false',
            permissionForSmallIntestine: 'false',
            permissionForSkin: 'false',
            requiresRelativeAcceptance: 'false',
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

async function DeleteOrganDonorRegistrationAdmin(attach, PatientCPR) {
    const body = {
        id: 'odrAdmin.DeleteOrganDonorRegistration',
        role: 'test.person',
        endpoint: env,
        fields: {
            CPR: PatientCPR,
        },
        'auth-attributes': {},
    };
    await drgWithAttach(attach, body);
}

module.exports = {
    CreateOrganDonorRegistration,
    GetOrganDonorRegistration,
    HasOrganDonorRegistration,
    DeleteOrganDonorRegistration,
    GetOrganDonorRegistrationAdmin,
    CreateOrganDonorRegistrationAdmin,
    DeleteOrganDonorRegistrationAdmin,
    UpdateOrganDonorRegistrationAdmin,
};
