const { env, drgWithAttach } = require('./util');

async function GetPersonalDataCard(attach, PatientCPR) {
    //Data til kald af SKR dannes
    const body = {
        id: 'skr.GetPersonalDataCard',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': { cpr: PatientCPR },
    };

    let SKR = await drgWithAttach(attach, body);

    let entry =
        SKR.body.response.main.Envelope.Body.GetPersonalDataCardResponse
            .ClinicalDocument.component.structuredBody.component.section.entry;
    return entry;
}

async function GetPersonalDataCardInvalidCpr(
    attach,
    PatientCPR,
    InvalidPatientCPR
) {
    //Data til kald af SKR dannes
    const body = {
        id: 'skr.GetPersonalDataCard',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
        },
        'auth-attributes': { cpr: InvalidPatientCPR },
    };
    console.log(body);

    let response = await drgWithAttach(attach, body);

    return response;
}

async function DeleteRelatives(attach, PatientCPR, PaaroerendeId) {
    const body = {
        id: 'skr.DeleteRelatives',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            relatedPersonId: PaaroerendeId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function UpdateRelatives(attach, PatientCPR, PaaroerendeId) {
    const body = {
        id: 'skr.UpdateRelatives',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            relatedPersonGivenName: 'Hans',
            relatedPersonFamilyName: 'Hansen',
            relationshipType: 'samboende',
            phoneH: '12121212',
            phoneWP: '23232323',
            phoneMC: '34343434',
            customAddress: 'Nej',
            customRelatedPersonId: 'Ja',
            relatedPersonId: PaaroerendeId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };
    const response = await drgWithAttach(attach, body);

    return response;
}

async function CreateRelative(attach, PatientCPR, RelativeId) {
    const body = {
        id: 'skr.CreateRelatives',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            relatedPersonGivenName: 'Peter oprindeligt navn',
            relatedPersonFamilyName: 'LOKtest',
            relationshipType: 'mor',
            phoneH: '12121212',
            phoneWP: '23232323',
            phoneMC: '34343434',
            customAddress: 'Nej',
            customRelatedPersonId: 'Ja',
            customUUID: RelativeId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function CreateTemporaryAddress(attach, TempAdressId, PatientCPR) {
    const body = {
        id: 'skr.CreateTemporaryAddress',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            streetAddressLine: 'Regressionstestvej 1',
            postalCode: '8000',
            city: 'Århus C',
            country: 'Denmark',
            useablePeriod1: '20201106',
            useablePeriod2: '20211108',
            customTemporaryAddressId: 'Ja',
            customUUID: TempAdressId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}
async function DeleteTemporaryAddress(attach, TempAdressId, PatientCPR) {
    const body = {
        id: 'skr.DeleteTemporaryAddress',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            temporaryAddressId: TempAdressId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}
async function UpdateTemporaryAddress(attach, TempAdressId, PatientCPR) {
    const body = {
        id: 'skr.UpdateTemporaryAddress',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            streetAddressLine: 'Opdateringsvej 10',
            postalCode: '8600',
            city: 'Silkeborg',
            country: 'Denmark',
            useablePeriod1: '20210106',
            useablePeriod2: '20210108',
            temporaryAddressId: TempAdressId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function CreateHealthProvider(attach, HealthProviderId, PatientCPR) {
    const body = {
        id: 'skr.CreateHealthProvider',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            yderNumber: '132435',
            yderName: 'Peter Jensen',
            phoneWP: '66666666',
            customAddress: 'Ja',
            streetAddressLine: 'Testvej',
            postalCode: '8660',
            city: 'Skanderborg',
            country: 'Denmark',
            customHealthProviderId: 'Ja',
            customUUID: HealthProviderId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function DeleteHealthProvider(attach, HealtProviderId, PatientCPR) {
    const body = {
        id: 'skr.DeleteHealthProvider',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            healthProviderId: HealtProviderId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function UpdateHealthProvider(attach, HealtProviderId, PatientCPR) {
    const body = {
        id: 'skr.UpdateHealthProvider',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            yderNumber: '132435',
            yderName: 'Mogens Hansen',
            phoneWP: '77777777',
            customAddress: 'Ja',
            streetAddressLine: 'Nytestvej',
            postalCode: '8000',
            city: 'Århus C',
            country: 'Denmark',
            healthProviderId: HealtProviderId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",

        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function CreateLanguage(attach, LanguageId, PatientCPR) {
    const body = {
        id: 'skr.CreateLanguage',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            languageCode: 'da',
            customLanguageId: 'Ja',
            customUUID: LanguageId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function DeleteLanguage(attach, LanguageId, PatientCPR) {
    const body = {
        id: 'skr.DeleteLanguage',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            languageId: LanguageId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function UpdateLanguage(attach, LanguageId, PatientCPR) {
    const body = {
        id: 'skr.UpdateLanguage',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            languageCode: 'en',
            languageId: LanguageId,
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    console.log(body);

    const response = await drgWithAttach(attach, body);

    return response;
}

async function UpdateContactInformation(attach, PatientCPR) {
    const body = {
        id: 'skr.UpdateContactInformation',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: PatientCPR,
            phoneH: '12345678',
            phoneWP: '12345678',
            phoneMC: '12345678',
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

async function UpdateContactInformationInvalidCpr(
    attach,
    PatientCPR,
    InvalidPatientCPR
) {
    const body = {
        id: 'skr.UpdateContactInformation',
        role: 'test.citizen',
        endpoint: env,
        fields: {
            cprNumber: InvalidPatientCPR,
            phoneH: '12345678',
            phoneWP: '12345678',
            phoneMC: '12345678',
            dataEntererFamily: "Family",
            dataEntererGiven: "Given",
        },
        'auth-attributes': {
            cpr: PatientCPR,
        },
    };

    const response = await drgWithAttach(attach, body);

    return response;
}

module.exports = {
    GetPersonalDataCard,
    GetPersonalDataCardInvalidCpr,
    DeleteRelatives,
    CreateRelative,
    UpdateRelatives,
    CreateTemporaryAddress,
    DeleteTemporaryAddress,
    UpdateTemporaryAddress,
    CreateHealthProvider,
    DeleteHealthProvider,
    UpdateHealthProvider,
    CreateLanguage,
    DeleteLanguage,
    UpdateLanguage,
    UpdateContactInformation,
    UpdateContactInformationInvalidCpr,
};
