const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect } = require('../util');
const {
    HasTreatmentWill,
    DeleteTreatmentWill,
    GetTreatmentWill,
    CreateTreatmentWill,
} = require('dataaccess/btr');
const roles = new Map(require('../../config/roles.json'));

// This fixture creates or deletes treatment will based on the status being either 'exists' or 'exists not'
Given(
    'behandlingstestamente {string} for borger med {string}',
    async function (Status, PatientCPR) {
        // Convert CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Calling the NAS Has Treatment Will to see if the patient has a treatment will and saves the response
        const hasTreatmentWill = await HasTreatmentWill(
            this.attach,
            PatientCPR
        );

        switch (Status) {
            // If the status is 'exists'
            case 'findes':
                // If the treatment will does not exist
                if (hasTreatmentWill == 'false') {
                    // Create new treatment will
                    await CreateTreatmentWill(this.attach, PatientCPR);
                }
                // else - treatment will exists
                break;
            // If the status is 'exists not'
            case 'findes ikke':
                // If the treatment will does exist
                if (hasTreatmentWill == 'true') {
                    // Delete treatment will
                    await DeleteTreatmentWill(this.attach, PatientCPR);
                } // else - treatment will does not exist
                break;
            default:
                break;
        }
    }
);

// This fixture expects the patient to have a treatment will
Then(
    'borger med {string} har behandlingstestamente',
    async function (PatientCPR) {
        // Convert CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Calling NAS Has Treatment Will Service and saves response
        const hasTreatmentWill = await HasTreatmentWill(
            this.attach,
            PatientCPR
        );

        // Expects the response to be true
        expect(hasTreatmentWill).to.eql('true');
    }
);

// This fixture expects a patient to have a treatment will with the data table
Then(
    'behandlingstestamente for borger med {string} har datasæt {string}',
    async function (PatientCPR, Dataset) {
        // Convert CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting dataset
        const dataset = this.datasets.get(Dataset);

        // Getting treatment will
        const treatmentWill = await GetTreatmentWill(this.attach, PatientCPR);

        // Expecting the treatment will to contain the data from the data table
        //No Life Proloning If Dying
        expect(treatmentWill.noLifeProlongingIfDying._text).to.eql(
            dataset.noLifeProlongingIfDying
        );

        //No Life Prolonging If Severely Degraded
        expect(treatmentWill.noLifeProlongingIfSeverelyDegraded._text).to.eql(
            dataset.noLifeProlongingIfSeverelyDegraded
        );

        //No Life Proloning If Severe Pain
        expect(treatmentWill.noLifeProlongingIfSeverePain._text).to.eql(
            dataset.noLifeProlongingIfSeverePain
        );

        //No Forced Treatment If Incapable
        expect(treatmentWill.noForcedTreatmentIfIncapable._text).to.eql(
            dataset.noForcedTreatmentIfIncapable
        );
    }
);
