const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { expect, findPathsAndValues } = require('../util');
const jq = require('node-jq');
const { v4 } = require('uuid');
const {
    ddsregistry: { getPDCEntries, getDDSRegistryWithMoces },
    skr: {
        CreateTemporaryAddress,
        DeleteTemporaryAddress,
        UpdateTemporaryAddress,
        CreateHealthProvider,
        DeleteHealthProvider,
        UpdateHealthProvider,
        CreateLanguage,
        DeleteLanguage,
        UpdateLanguage,
        CreateRelative,
        DeleteRelatives,
        UpdateRelatives,
        GetPersonalDataCard,
    },
    odr: {
        CreateOrganDonorRegistration,
        GetOrganDonorRegistration,
        HasOrganDonorRegistration,
        DeleteOrganDonorRegistration,
        GetOrganDonorRegistrationAdmin,
        CreateOrganDonorRegistrationAdmin,
        DeleteOrganDonorRegistrationAdmin,
        UpdateOrganDonorRegistrationAdmin,
    },
    btr: {
        CreateTreatmentWill,
        DeleteTreatmentWill,
        GetTreatmentWill,
        CreateTreatmentWillAdmin,
        DeleteTreatmentWillAdmin,
        UpdateTreatmentWillAdmin,
        GetTreatmentWillAdmin,
    },
} = require('dataaccess');
const roles = new Map(require('../../config/roles.json'));

// This fixture expects registry to contain a given number of adding to MinLog
Then(
    'kaldet har registreret præcist {int} tilføjelse til Minlog',
    function (count) {
        // Expecting registry to contain a given number of adding to MinLog
        expect(
            this.getLastResponse().body.response.main.Envelope.Body
                .RegistrationResponse.NumberAdded._text
        ).to.eql('' + count);
    }
);

// This fixture expects a logging in MinLog to equal data from data table
Then('kaldet indeholder en logning med datasæt {string}', function (name) {
    // Getting data table
    const dataset = this.datasets.get(name);

    // Paths to check in logging
    const checkMapning = {
        correlationId: 'CorrelationId._text',
        cprNumber: 'PersonIdentifier._text',
        dateTime: 'DateTime._text',
        orgName: 'OrganisationName._text',
        orgId: 'OrganisationId._text',
        orgIdSource: 'OrganisationId.__source',
        userCprNumber: 'UserPersonIdentifier._text',
    };

    // Getting entry from last response
    let entry =
        this.getLastResponse().body.response.main.Envelope.Body
            .LogStatementsResponse.LogDataEntry;

    // If entry is an array, finding the right entry
    if (Array.isArray(entry)) {
        entry = entry[entry.length - 1];
    }
    // Finding paths and values in entry
    const flat = findPathsAndValues(entry.Destination);

    // Expecting elements in dataset to equal elements in entry
    for (const { k, v } of Object.entries(dataset)) {
        expect(dataset[k]).to.eql(flat[checkMapning[k]]);
    }
});

Then(
    'kaldet indeholder en tekst logning med {string} lig {string}',
    function (field, value) {
        //Alle kald indenfor et minut kommer tilsyneladende i tilfældig orden
        //Tager næstsidste logning, da den sidste er kaldet til MinLog selv
        tempValue = value

        value = this.replaceVariables(value);
        let entry =
            this.getLastResponse().body.response.main.Envelope.Body
                .LogStatementsResponse.LogDataEntry;
        console.log(typeof entry);
        console.log(entry instanceof Array);
        console.log(JSON.stringify(entry, null, 2));

        if (Array.isArray(entry)) {
            let entryOld = entry;

            entry = entry[entry.length - 1];
            let systemName = entry.Destination.SystemName._text;
            if (systemName == 'MinLog') {
                entry = entryOld[entryOld.length - 2];
            }
        }

        if (value == 'Now') {
            value = new Date();
            bodyDate = new Date(entry.Destination[field]._text).toISOString();
            expect(Date.parse(new Date(bodyDate))).to.be.within(
                Date.parse(value) - 120000,
                Date.parse(value)
            );
            // Fjerner millisek da nogle services herunder BRS/MinLog2 ikke kan lide dem
        } else if (tempValue == 'Now') {
            expect(entry.Destination[field]._text.toISOString().replace(/\.\d+Z/, 'Z')).to.eql(value);
        } else {
            expect(entry.Destination[field]._text).to.eql(value);
        }
    }
);

// This fixtue makes sure to call SKR data card service
Given(
    'der er lavet et opslag på SKR dataCard for {string}',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Calling service to get PDC
        await GetPersonalDataCard(this.attach, PatientCPR);
    }
);

// This fixture makes sure to call DDS registry with MOCES with a specific role
Given(
    'rolle {string} lige har hentet DDS registry for borger {string}',
    async function (Role, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Calling service to get DDS Registry by specific role
        await getDDSRegistryWithMoces(this.attach, Role, PatientCPR);
    }
);

//#region Temporary Address
// This fixtures creates temporary address in SKR
Given(
    'midlertidig adresse er oprettet i SKR for {string}',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Generating temporary address id
        let tempAdressId = v4();

        // Creating temporary address
        await CreateTemporaryAddress(this.attach, tempAdressId, PatientCPR);
    }
);

// This fixtures deletes temporary address in SKR
Given(
    'midlertidig adresse er slettet i SKR for {string}',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC entries
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

        // Finding pdc component with temporary address
        const pdcComponent = await jq.run(
            `.[]|select(.observation.code.__code=="TempAddrTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Deleting temporary address
        await DeleteTemporaryAddress(
            this.attach,
            pdcComponent.observation.id.__extension,
            PatientCPR
        );
    }
);

// This fixtures updates temporary address in SKR
Given(
    'midlertidig adresse er opdateret i SKR for {string}',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting PDC entries
        let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

        // Finding pdc component with temporary address
        const pdcComponent = await jq.run(
            `.[]|select(.observation.code.__code=="TempAddrTypedIn")`,
            pdcEntries,
            { input: 'json', output: 'json' }
        );

        // Updating temporary address
        await UpdateTemporaryAddress(
            this.attach,
            pdcComponent.observation.id.__extension,
            PatientCPR
        );
    }
);
//#endregion

//#region Health Provider
// This fixtures creates health provider in SKR
Given('tandlæge er oprettet i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Generating health provider id
    let healthProviderId = v4();

    // Creating health provider
    await CreateHealthProvider(this.attach, healthProviderId, PatientCPR);
});

// This fixtures deletes health provider in SKR
Given('tandlæge er slettet i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Getting PDC entries
    let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

    // Finding pdc component with health provider
    const pdcComponent = await jq.run(
        `.[]|select(.observation.code.__code=="DentistTypedIn")`,
        pdcEntries,
        { input: 'json', output: 'json' }
    );

    // Deleting health provider
    await DeleteHealthProvider(
        this.attach,
        pdcComponent.observation.id.__extension,
        PatientCPR
    );
});

// This fixtures updates health provider in SKR
Given('tandlæge er opdateret i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Getting PDC entries
    let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

    // Finding pdc component with health provider
    const pdcComponent = await jq.run(
        `.[]|select(.observation.code.__code=="DentistTypedIn")`,
        pdcEntries,
        { input: 'json', output: 'json' }
    );

    // Updating health provider
    await UpdateHealthProvider(
        this.attach,
        pdcComponent.observation.id.__extension,
        PatientCPR
    );
});
//#endregion

//#region Language
// This fixtures creates language in SKR
Given('sprog er oprettet i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Generating language id
    let LanguageId = v4();

    // Creating language
    await CreateLanguage(this.attach, LanguageId, PatientCPR);
});

// This fixtures deletes language in SKR
Given('sprog er slettet i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Getting PDC entries
    let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

    // Finding pdc component with language
    const pdcComponent = await jq.run(
        `.[]|select(.observation.code.__code=="LanguageTypedIn")`,
        pdcEntries,
        { input: 'json', output: 'json' }
    );

    // Deleting language
    await DeleteLanguage(
        this.attach,
        pdcComponent.observation.id.__extension,
        PatientCPR
    );
});

// This fixtures updates language in SKR
Given('sprog er opdateret i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Getting PDC entries
    let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

    // Finding pdc component with language
    const pdcComponent = await jq.run(
        `.[]|select(.observation.code.__code=="LanguageTypedIn")`,
        pdcEntries,
        { input: 'json', output: 'json' }
    );

    // Updating language
    await UpdateLanguage(
        this.attach,
        pdcComponent.observation.id.__extension,
        PatientCPR
    );
});
//#endregion

//#region Related Person
// This fixtures creates related person in SKR
Given('pårørende er oprettet i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Generating related person id
    let RelatedPersonId = v4();

    // Creating related person
    await CreateRelative(this.attach, PatientCPR, RelatedPersonId);
});

// This fixtures deletes related person in SKR
Given('pårørende er slettet i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Getting PDC entries
    let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

    // Finding pdc component with related person
    const pdcComponent = await jq.run(
        `.[]|select(.observation.code|select(."__code"? =="RelativeTypedIn"))`,
        pdcEntries,
        { input: 'json', output: 'json' }
    );

    // Deleting related person
    await DeleteRelatives(
        this.attach,
        PatientCPR,
        pdcComponent.observation.id.__extension
    );
});

// This fixtures updates related person in SKR
Given('pårørende er opdateret i SKR for {string}', async function (PatientCPR) {
    // Replacing CPR
    PatientCPR = this.replaceVariables(PatientCPR);

    // Getting PDC entries
    let pdcEntries = await getPDCEntries(this.attach, PatientCPR);

    // Finding pdc component with related person
    const pdcComponent = await jq.run(
        `.[]|select(.observation.code|select(."__code"? =="RelativeTypedIn"))`,
        pdcEntries,
        { input: 'json', output: 'json' }
    );

    // Updating related person
    await UpdateRelatives(
        this.attach,
        PatientCPR,
        pdcComponent.observation.id.__extension
    );
});
//#endregion

//#region TreatmenWill
// This fixtures creates treatment will
Given(
    'behandlingstestamente er oprettet for {string} af borger',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating treatment will
        await CreateTreatmentWill(this.attach, PatientCPR);
    }
);

// This fixtures deletes treatment will
Given(
    'behandlingstestamente er slettet for {string} af borger',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Deleting treatment will
        await DeleteTreatmentWill(this.attach, PatientCPR);
    }
);

// This fixtures gets treatment will
Given(
    'behandlingstestamente er hentet for {string} af borger',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting treatment will
        await GetTreatmentWill(this.attach, PatientCPR);
    }
);

// This fixtures creates treatment will (ADMIN)
Given(
    'behandlingstestamente er oprettet for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating treatment will
        await CreateTreatmentWillAdmin(this.attach, PatientCPR);
    }
);

// This fixtures creates treatment will (ADMIN)
Given(
    'behandlingstestamente er slettet for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Deleting treatment will
        await DeleteTreatmentWillAdmin(this.attach, PatientCPR);
    }
);

// This fixtures creates treatment will (ADMIN)
Given(
    'behandlingstestamente er opdateret for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Updating treatment will
        await UpdateTreatmentWillAdmin(this.attach, PatientCPR);
    }
);

// This fixtures creates treatment will (ADMIN)
Given(
    'behandlingstestamente er hentet for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting treatment will
        await GetTreatmentWillAdmin(this.attach, PatientCPR);
    }
);
//#endregion

//#region OrganDonor
// This fixture makes sure that a patient has og has not a organ donor registration based on the status
// tilføjet timeout for at testcase 433
Given(
    'organdonorregistrering {string} for borger med {string}',
    { timeout: -1 },
    async function (Status, PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Checking if the patient has a organ donor registration
        const hasOrganDonor = await HasOrganDonorRegistration(
            this.attach,
            PatientCPR
        );

        // Switch cases based on organ donor registration must or must not exist
        switch (Status) {
            case 'findes':
                // No organ donor registration exists
                if (hasOrganDonor == 'false') {
                    // Creating organ donor registration
                    await CreateOrganDonorRegistration(this.attach, PatientCPR);
                }
                break;
            case 'findes ikke':
                // Organ donor registration exists
                if (hasOrganDonor == 'true') {
                    // Deleting organ donor registration
                    await DeleteOrganDonorRegistration(this.attach, PatientCPR);
                }
                break;
            default:
                // Throwing error if status is unknown
                throw new Error(`Unknown status "${Status}"`);
        }
    }
);

// This fixture gets organ donor registration
Given(
    'organdonorregistrering er hentet for {string} af borger',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting organ donor registration
        await GetOrganDonorRegistration(this.attach, PatientCPR);
    }
);

// This fixture creates organ donor registration
Given(
    'organdonorregistrering er oprettet for {string} af borger',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating organ donor registration
        await CreateOrganDonorRegistration(this.attach, PatientCPR);
    }
);

// This fixture creates organ donor registration (ADMIN)
Given(
    'organdonorregistrering er oprettet for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Creating organ donor registration
        await CreateOrganDonorRegistrationAdmin(this.attach, PatientCPR);
    }
);

// This fixture deletes organ donor registration
Given(
    'organdonorregistrering er slettet for {string} af borger',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Deleting organ donor registration
        await DeleteOrganDonorRegistration(this.attach, PatientCPR);
    }
);

// This fixture deletes organ donor registration (ADMIN)
Given(
    'organdonorregistrering er slettet for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Deleting organ donor registration
        await DeleteOrganDonorRegistrationAdmin(this.attach, PatientCPR);
    }
);

// This fixture updates organ donor registration
Given(
    'organdonorregistrering er opdateret for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Updating organ donor registration
        await UpdateOrganDonorRegistrationAdmin(this.attach, PatientCPR);
    }
);

// This fixtures gets organ donor registration (ADMIN)
Given(
    'organdonorregistrering er hentet for {string} af Admin',
    async function (PatientCPR) {
        // Replacing CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting treatment will
        await GetOrganDonorRegistrationAdmin(this.attach, PatientCPR);
    }
);
//#endregion
