const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const jq = require('node-jq');
const { expect } = require('../util');
const {
    GetOrganDonorRegistration,
    HasOrganDonorRegistration,
} = require('dataaccess/odr');
const roles = new Map(require('../../config/roles.json'));

// This fixture expects a patient to have an organ donor registration
Then(
    'borger med {string} har organdonorregistrering',
    async function (PatientCPR) {
        // Convert CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Calls NAS Has Organ Donor Registration Service and saves response
        const hasOrganDonorRegistration = await HasOrganDonorRegistration(
            this.attach,
            PatientCPR
        );

        // Expects the response to be true
        expect(hasOrganDonorRegistration).to.eql('true');
    }
);

// This fixture expects a patient to have an organ donor registration with a given data table
Then(
    'organdonorregistrering for borger med {string} har datasæt {string}',
    async function (PatientCPR, Dataset) {
        // Convert CPR
        PatientCPR = this.replaceVariables(PatientCPR);

        // Getting dataset
        const dataset = this.datasets.get(Dataset);

        // Getting organ donor registration and saving it
        const organDonorRegistration = await GetOrganDonorRegistration(
            this.attach,
            PatientCPR
        );

        // If the datasets permission type is...
        if (
            dataset.permissionType == 'LIMITED_WITH_RESEARCH' ||
            dataset.permissionType == 'LIMITED'
        ) {
            //Permission Type
            expect(organDonorRegistration.permissionType._text).to.eql(
                dataset.permissionType
            );

            //Permission For Heart
            expect(organDonorRegistration.permissionForHeart._text).to.eql(
                dataset.permissionForHeart
            );

            //Permission For Lungs
            expect(organDonorRegistration.permissionForLungs._text).to.eql(
                dataset.permissionForLungs
            );

            //Permission For Liver
            expect(organDonorRegistration.permissionForLiver._text).to.eql(
                dataset.permissionForLiver
            );

            //Permission For Pancreas
            expect(organDonorRegistration.permissionForPancreas._text).to.eql(
                dataset.permissionForPancreas
            );

            //Permission For Kidneys
            expect(organDonorRegistration.permissionForKidneys._text).to.eql(
                dataset.permissionForKidneys
            );

            //Permission For Cornea
            expect(organDonorRegistration.permissionForCornea._text).to.eql(
                dataset.permissionForCornea
            );

            //Permission For Small Intestine
            expect(
                organDonorRegistration.permissionForSmallIntestine._text
            ).to.eql(dataset.permissionForSmallIntestine);

            //Permission For Skin
            expect(organDonorRegistration.permissionForSkin._text).to.eql(
                dataset.permissionForSkin
            );
        } else if (
            !(
                dataset.permissionType == 'NONE' ||
                dataset.permissionType == 'DONT_KNOW'
            )
        ) {
            //Requires Relative Acceptance
            expect(
                organDonorRegistration.requiresRelativeAcceptance._text
            ).to.eql(dataset.requiresRelativeAcceptance);
        }
    }
);
