const {
    Before,
    BeforeAll,
    After,
    Given,
    Then,
    When,
} = require('@cucumber/cucumber');
const { expect } = require('../util');
const jq = require('node-jq');

// This fixture expects yder in response to equal value
Then('yder indeholder {string} lig med {string}', function (Field, Value) {
    // Getting yder from last response
    const response =
        this.getLastResponse().body.response.main.Envelope.Body
            .YderResponseStructure.yder;

    // Expecting yder-response-field to equal value
    expect(response[Field]._text).to.eql(Value);
});

// This fixture counts persons attached to the yder
Then('der er {int} personer tilknyttet yder', function (NumberOfPersons) {
    // Getting persons from last response
    const response =
        this.getLastResponse().body.response.main.Envelope.Body
            .YderResponseStructure.person;

    // Counting persons and expecting the right number of persons
    expect(response.length).to.eql(NumberOfPersons);
});

// This fixture expects yder person to contain data from data table
Then(
    'yderperson med cpr {string} indeholder data',
    async function (CPR, dataTable) {
        // Getting persons from last response
        const response =
            this.getLastResponse().body.response.main.Envelope.Body
                .YderResponseStructure.person;

        // Parsing data table
        const data = this.parseDataTable(dataTable);

        // Finding person with CPR
        let person = await jq.run(
            `.[]|select(.CPR._text == "${CPR}")`,
            response,
            { input: 'json', output: 'json' }
        );

        // Expecting persons dates to equal dates from data table
        expect(person.Tilgangsdato._text).to.equal(data.Tilgangsdato);
        expect(person.Afgangsdato._text).to.equal(data.Afgangsdato);

        // Expectiong person role code and text to equal value in data table
        if (data.PersonrolleKode != undefined) {
            expect(person.PersonrolleKode._text).to.equal(data.PersonrolleKode);
        }
        if (data.PersonrolleTekst != undefined) {
            expect(person.PersonrolleTekst._text).to.equal(
                data.PersonrolleTekst
            );
        }
    }
);
